@@ -27,7 +27,7 @@
  * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php">Atom specification</ulink>.
  *
  * Since: 0.4.0
- **/
+ */
 
 #include <glib.h>
 #include <libxml/parser.h>
@@ -91,7 +91,7 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.link">Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_URI,
 	                                 g_param_spec_string ("uri",
 	                                                      "URI", "The link's IRI.",
@@ -107,7 +107,7 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.link">Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_RELATION_TYPE,
 	                                 g_param_spec_string ("relation-type",
 	                                                      "Relation type", "The link relation type.",
@@ -124,7 +124,7 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.link">Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_CONTENT_TYPE,
 	                                 g_param_spec_string ("content-type",
 	                                                      "Content type", "An advisory media type.",
@@ -140,7 +140,7 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.link">Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_LANGUAGE,
 	                                 g_param_spec_string ("language",
 	                                                      "Language", "Describes the language of the resource pointed to by the uri property.",
@@ -156,7 +156,7 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.link">Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_TITLE,
 	                                 g_param_spec_string ("title",
 	                                                      "Title", "Conveys human-readable information about the link.",
@@ -172,7 +172,7 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.link">Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_LENGTH,
 	                                 g_param_spec_int ("length",
 	                                                   "Length", "Indicates an advisory length of the linked content in octets.",
@@ -293,38 +293,41 @@ pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointe
 		xmlFree (uri);
 		return gdata_parser_error_required_property_missing (root_node, "href", error);
 	}
-	self->priv->uri = (gchar*) uri;
 
 	/* rel */
 	relation_type = xmlGetProp (root_node, (xmlChar*) "rel");
 	if (relation_type != NULL && *relation_type == '\0') {
+		xmlFree (uri);
 		xmlFree (relation_type);
 		return gdata_parser_error_required_property_missing (root_node, "rel", error);
 	}
 
-	gdata_link_set_relation_type (self, (const gchar*) relation_type);
-	xmlFree (relation_type);
-
 	/* type */
 	content_type = xmlGetProp (root_node, (xmlChar*) "type");
 	if (content_type != NULL && *content_type == '\0') {
+		xmlFree (uri);
+		xmlFree (relation_type);
 		xmlFree (content_type);
 		return gdata_parser_error_required_property_missing (root_node, "type", error);
 	}
-	self->priv->content_type = (gchar*) content_type;
 
 	/* hreflang */
 	language = xmlGetProp (root_node, (xmlChar*) "hreflang");
 	if (language != NULL && *language == '\0') {
+		xmlFree (uri);
+		xmlFree (relation_type);
+		xmlFree (content_type);
 		xmlFree (language);
 		return gdata_parser_error_required_property_missing (root_node, "hreflang", error);
 	}
-	self->priv->language = (gchar*) language;
 
-	/* title */
+	self->priv->uri = (gchar*) uri;
+	gdata_link_set_relation_type (self, (const gchar*) relation_type);
+	xmlFree (relation_type);
+	self->priv->content_type = (gchar*) content_type;
+	self->priv->language = (gchar*) language;
 	self->priv->title = (gchar*) xmlGetProp (root_node, (xmlChar*) "title");
 
-	/* length */
 	length = xmlGetProp (root_node, (xmlChar*) "length");
 	if (length == NULL)
 		self->priv->length = -1;
@@ -362,8 +365,10 @@ pre_get_xml (GDataParsable *parsable, GString *xml_string)
  * Creates a new #GDataLink. More information is available in the <ulink type="http"
  * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.link">Atom specification</ulink>.
  *
+ * @uri must be non-%NULL and non-empty. @relation_type must be %NULL or non-empty.
+ *
  * Return value: a new #GDataLink, or %NULL; unref with g_object_unref()
- **/
+ */
 GDataLink *
 gdata_link_new (const gchar *uri, const gchar *relation_type)
 {
@@ -387,7 +392,7 @@ gdata_link_new (const gchar *uri, const gchar *relation_type)
  * Return value: the link's URI
  *
  * Since: 0.4.0
- **/
+ */
 const gchar *
 gdata_link_get_uri (GDataLink *self)
 {
@@ -400,10 +405,10 @@ gdata_link_get_uri (GDataLink *self)
  * @self: a #GDataLink
  * @uri: the new URI for the link
  *
- * Sets the #GDataLink:uri property to @uri.
+ * Sets the #GDataLink:uri property to @uri. @uri must be non-%NULL and non-empty.
  *
  * Since: 0.4.0
- **/
+ */
 void
 gdata_link_set_uri (GDataLink *self, const gchar *uri)
 {
@@ -419,12 +424,12 @@ gdata_link_set_uri (GDataLink *self, const gchar *uri)
  * gdata_link_get_relation_type:
  * @self: a #GDataLink
  *
- * Gets the #GDataLink:relation-type property.
+ * Gets the #GDataLink:relation-type property. If the relation type is non-%NULL, it will be non-empty.
  *
- * Return value: the link's relation type
+ * Return value: (nullable): the link's relation type
  *
  * Since: 0.4.0
- **/
+ */
 const gchar *
 gdata_link_get_relation_type (GDataLink *self)
 {
@@ -443,7 +448,7 @@ gdata_link_get_relation_type (GDataLink *self)
  * Set @relation_type to %NULL to unset the property in the link.
  *
  * Since: 0.4.0
- **/
+ */
 void
 gdata_link_set_relation_type (GDataLink *self, const gchar *relation_type)
 {
@@ -469,12 +474,12 @@ gdata_link_set_relation_type (GDataLink *self, const gchar *relation_type)
  * gdata_link_get_content_type:
  * @self: a #GDataLink
  *
- * Gets the #GDataLink:content-type property.
+ * Gets the #GDataLink:content-type property. If the content type is non-%NULL, it will be non-empty.
  *
- * Return value: the link's content type, or %NULL
+ * Return value: (nullable): the link's content type, or %NULL
  *
  * Since: 0.4.0
- **/
+ */
 const gchar *
 gdata_link_get_content_type (GDataLink *self)
 {
@@ -487,12 +492,12 @@ gdata_link_get_content_type (GDataLink *self)
  * @self: a #GDataLink
  * @content_type: (allow-none): the new content type for the link, or %NULL
  *
- * Sets the #GDataLink:content-type property to @content_type.
+ * Sets the #GDataLink:content-type property to @content_type. @content_type must be %NULL or non-empty.
  *
  * Set @content_type to %NULL to unset the property in the link.
  *
  * Since: 0.4.0
- **/
+ */
 void
 gdata_link_set_content_type (GDataLink *self, const gchar *content_type)
 {
@@ -508,12 +513,12 @@ gdata_link_set_content_type (GDataLink *self, const gchar *content_type)
  * gdata_link_get_language:
  * @self: a #GDataLink
  *
- * Gets the #GDataLink:language property.
+ * Gets the #GDataLink:language property. If the language is non-%NULL, it will be non-empty.
  *
- * Return value: the link's language, or %NULL
+ * Return value: (nullable): the link's language, or %NULL
  *
  * Since: 0.4.0
- **/
+ */
 const gchar *
 gdata_link_get_language (GDataLink *self)
 {
@@ -526,12 +531,12 @@ gdata_link_get_language (GDataLink *self)
  * @self: a #GDataLink
  * @language: (allow-none): the new language for the link, or %NULL
  *
- * Sets the #GDataLink:language property to @language.
+ * Sets the #GDataLink:language property to @language. @language must be %NULL or non-empty.
  *
  * Set @language to %NULL to unset the property in the link.
  *
  * Since: 0.4.0
- **/
+ */
 void
 gdata_link_set_language (GDataLink *self, const gchar *language)
 {
@@ -552,7 +557,7 @@ gdata_link_set_language (GDataLink *self, const gchar *language)
  * Return value: the link's title, or %NULL
  *
  * Since: 0.4.0
- **/
+ */
 const gchar *
 gdata_link_get_title (GDataLink *self)
 {
@@ -570,7 +575,7 @@ gdata_link_get_title (GDataLink *self)
  * Set @title to %NULL to unset the property in the link.
  *
  * Since: 0.4.0
- **/
+ */
 void
 gdata_link_set_title (GDataLink *self, const gchar *title)
 {
@@ -590,7 +595,7 @@ gdata_link_set_title (GDataLink *self, const gchar *title)
  * Return value: the link's length, or <code class="literal">-1</code>
  *
  * Since: 0.4.0
- **/
+ */
 gint
 gdata_link_get_length (GDataLink *self)
 {
@@ -608,7 +613,7 @@ gdata_link_get_length (GDataLink *self)
  * Set @length to <code class="literal">-1</code> to unset the property in the link.
  *
  * Since: 0.4.0
- **/
+ */
 void
 gdata_link_set_length (GDataLink *self, gint length)
 {
