@@ -76,7 +76,7 @@
  * </example>
  *
  * Since: 0.3.0
- **/
+ */
 
 #include <config.h>
 #include <glib.h>
@@ -95,6 +95,7 @@ static void get_xml (GDataParsable *parsable, GString *xml_string);
 static void gdata_access_rule_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static void gdata_access_rule_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error);
+static gboolean post_parse_xml (GDataParsable *parsable, gpointer user_data, GError **error);
 
 struct _GDataAccessRulePrivate {
 	gchar *role;
@@ -130,6 +131,7 @@ gdata_access_rule_class_init (GDataAccessRuleClass *klass)
 	gobject_class->set_property = gdata_access_rule_set_property;
 
 	parsable_class->parse_xml = parse_xml;
+	parsable_class->post_parse_xml = post_parse_xml;
 	parsable_class->get_xml = get_xml;
 	parsable_class->get_namespaces = get_namespaces;
 
@@ -142,7 +144,7 @@ gdata_access_rule_class_init (GDataAccessRuleClass *klass)
 	 * their own namespaced roles.
 	 *
 	 * Since: 0.3.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_ROLE,
 	                                 g_param_spec_string ("role",
 	                                                      "Role", "The role of the person concerned by this ACL.",
@@ -155,7 +157,7 @@ gdata_access_rule_class_init (GDataAccessRuleClass *klass)
 	 * Specifies to whom this access rule applies. For example, %GDATA_ACCESS_SCOPE_USER or %GDATA_ACCESS_SCOPE_DEFAULT.
 	 *
 	 * Since: 0.3.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_SCOPE_TYPE,
 	                                 g_param_spec_string ("scope-type",
 	                                                      "Scope type", "Specifies to whom this access rule applies.",
@@ -171,7 +173,7 @@ gdata_access_rule_class_init (GDataAccessRuleClass *klass)
 	 * This must be %NULL if and only if #GDataAccessRule:scope-type is %GDATA_ACCESS_SCOPE_DEFAULT.
 	 *
 	 * Since: 0.3.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_SCOPE_VALUE,
 	                                 g_param_spec_string ("scope-value",
 	                                                      "Scope value", "The scope value for this access rule.",
@@ -187,7 +189,7 @@ gdata_access_rule_class_init (GDataAccessRuleClass *klass)
 	 * Atom Publishing Protocol specification</ulink>.
 	 *
 	 * Since: 0.7.0
-	 **/
+	 */
 	g_object_class_install_property (gobject_class, PROP_EDITED,
 	                                 g_param_spec_int64 ("edited",
 	                                                     "Edited", "The last time the access rule was edited.",
@@ -362,16 +364,20 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		if (xmlStrcmp (node->name, (xmlChar*) "role") == 0) {
 			/* gAcl:role */
 			xmlChar *role = xmlGetProp (node, (xmlChar*) "value");
-			if (role == NULL)
+			if (role == NULL || *role == '\0') {
+				xmlFree (role);
 				return gdata_parser_error_required_property_missing (node, "value", error);
+			}
 			self->priv->role = (gchar*) role;
 		} else if (xmlStrcmp (node->name, (xmlChar*) "scope") == 0) {
 			/* gAcl:scope */
 			xmlChar *scope_type, *scope_value;
 
 			scope_type = xmlGetProp (node, (xmlChar*) "type");
-			if (scope_type == NULL)
+			if (scope_type == NULL || *scope_type == '\0') {
+				xmlFree (scope_type);
 				return gdata_parser_error_required_property_missing (node, "type", error);
+			}
 
 			scope_value = xmlGetProp (node, (xmlChar*) "value");
 
@@ -434,6 +440,20 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 	return TRUE;
 }
 
+static gboolean
+post_parse_xml (GDataParsable *parsable, gpointer user_data, GError **error)
+{
+	GDataAccessRulePrivate *priv = GDATA_ACCESS_RULE (parsable)->priv;
+
+	/* Check for missing required elements */
+	if (gdata_entry_get_title (GDATA_ENTRY (parsable)) == NULL || *gdata_entry_get_title (GDATA_ENTRY (parsable)) == '\0')
+		return gdata_parser_error_required_element_missing ("role", "entry", error);
+	if (priv->scope_type == NULL)
+		return gdata_parser_error_required_element_missing ("scope", "entry", error);
+
+	return TRUE;
+}
+
 static void
 get_xml (GDataParsable *parsable, GString *xml_string)
 {
@@ -489,7 +509,7 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
  * Return value: a new #GDataAccessRule; unref with g_object_unref()
  *
  * Since: 0.3.0
- **/
+ */
 GDataAccessRule *
 gdata_access_rule_new (const gchar *id)
 {
@@ -501,16 +521,17 @@ gdata_access_rule_new (const gchar *id)
  * @self: a #GDataAccessRule
  * @role: a new role, or %NULL
  *
- * Sets the #GDataAccessRule:role property to @role.
+ * Sets the #GDataAccessRule:role property to @role. @role must be a non-empty string, such as %GDATA_ACCESS_ROLE_NONE.
  *
  * Set @role to %NULL to unset the property in the access rule.
  *
  * Since: 0.3.0
- **/
+ */
 void
 gdata_access_rule_set_role (GDataAccessRule *self, const gchar *role)
 {
 	g_return_if_fail (GDATA_IS_ACCESS_RULE (self));
+	g_return_if_fail (role == NULL || *role != '\0');
 
 	g_free (self->priv->role);
 	self->priv->role = g_strdup (role);
@@ -526,7 +547,7 @@ gdata_access_rule_set_role (GDataAccessRule *self, const gchar *role)
  * Return value: the access rule's role, or %NULL
  *
  * Since: 0.3.0
- **/
+ */
 const gchar *
 gdata_access_rule_get_role (GDataAccessRule *self)
 {
@@ -551,12 +572,12 @@ gdata_access_rule_get_role (GDataAccessRule *self)
  * documentation</ulink> for more information.
  *
  * Since: 0.3.0
- **/
+ */
 void
 gdata_access_rule_set_scope (GDataAccessRule *self, const gchar *type, const gchar *value)
 {
 	g_return_if_fail (GDATA_IS_ACCESS_RULE (self));
-	g_return_if_fail (type != NULL);
+	g_return_if_fail (type != NULL && *type != '\0');
 	g_return_if_fail ((strcmp (type, GDATA_ACCESS_SCOPE_DEFAULT) == 0 && value == NULL) || value != NULL);
 
 	g_free (self->priv->scope_type);
@@ -580,7 +601,7 @@ gdata_access_rule_set_scope (GDataAccessRule *self, const gchar *type, const gch
  * Gets the #GDataAccessRule:scope-type and #GDataAccessRule:scope-value properties.
  *
  * Since: 0.3.0
- **/
+ */
 void
 gdata_access_rule_get_scope (GDataAccessRule *self, const gchar **type, const gchar **value)
 {
@@ -600,7 +621,7 @@ gdata_access_rule_get_scope (GDataAccessRule *self, const gchar **type, const gc
  * Return value: the UNIX timestamp for the time the access rule was last edited, or <code class="literal">-1</code>
  *
  * Since: 0.7.0
- **/
+ */
 gint64
 gdata_access_rule_get_edited (GDataAccessRule *self)
 {
